#include "artstubs.h"
#include "svp_helper.h"

/********************************************************************/
/* blending operations                                              */
/********************************************************************/

/* ggi_color-based blending - c2 over c1 */
static inline void
blend_color(ggi_color *dst, ggi_color *c1, ggi_color *c2) {
  dst->r=c1->r+(((c2->r-c1->r)*(c2->a >> 8)) >> 8);
  dst->g=c1->g+(((c2->g-c1->g)*(c2->a >> 8)) >> 8);
  dst->b=c1->b+(((c2->b-c1->b)*(c2->a >> 8)) >> 8);
  /*
    If we want to keep the opacity later, I guees it is the highest of the two.
    dst->a=max(c1->a,c2->a);
  */
}

/*****************************************************************/
/* rendering callbacks                                           */
/*****************************************************************/

#include "rops/svp_color.ROP"
#include "rops/svp_not.ROP"
#include "rops/svp_xor.ROP"
#include "rops/svp_and.ROP"
#include "rops/svp_or.ROP"
#include "rops/svp_texture.ROP"

/*****************************************************************/
/* Actual stubs                                                  */
/*****************************************************************/

/***************** SVP ***********************/
void
GGIART_stubs_render_svp(ggi_visual_t vis,
			const ggiart_svp_t svp,
			const ggiart_uta_t bbox,
			const ggiart_rop_t rop) {
  
  switch(rop->rop) {
  case GGIART_ROP_COLOR:
    ROP_svp_color(vis,svp,bbox,(ggi_color*)rop->arg1);
    break;
  case GGIART_ROP_NOT:
    ROP_svp_not(vis,svp,bbox,(int)rop->arg1);
    break;
  case GGIART_ROP_XOR:
    ROP_svp_xor(vis,svp,bbox,(int)rop->arg1,(ggi_pixel)rop->arg2);
    break;
  case GGIART_ROP_AND:
    ROP_svp_and(vis,svp,bbox,(int)rop->arg1,(ggi_pixel)rop->arg2);
    break;
  case GGIART_ROP_OR:
    ROP_svp_or(vis,svp,bbox,(int)rop->arg1,(ggi_pixel)rop->arg2);
    break;
  case GGIART_ROP_TEXTURE:
    ROP_svp_texture(vis,svp,bbox,(ggiart_buffer_t)rop->arg1,(int *)rop->arg2);
  default:
    GGIARTDPRINT("GGIART_stubs_render_svp ROP %i not implemented.\n",rop->rop);
  }
}

/******************* buffer *************************/
void
GGIART_stubs_render_buffer(ggi_visual_t vis,
			   const ggiart_buffer_t buffer,
			   const double affine[6],
			   const ggiart_uta_t uta,
			   const int alpha) {
  GGIARTDPRINT("GGIART_stubs_render_buffer not implemented.\n");
}



/******************* fillscreen *********************/
void
GGIART_stubs_fillscreen(ggi_visual_t vis, const ggiart_uta_t uta) {
  int w,h,x,y;
  ArtUtaBbox ub;
  
  /* if bbox is null, clear the whole buffer */
  if(!uta) {
    ggiFillscreen(vis);
  }
  /* else, run over the region covered by the uta */
  else {
    w=uta->width;
    h=uta->height;
    for (y = 0; y < h; y++)
      for (x = 0; x < w; x++) {
	ub = uta->utiles[y * w + x];
	if (ub) {
	  ggiDrawBox(vis,
		     (uta->x0 + x) * ART_UTILE_SIZE + ART_UTA_BBOX_X0(ub),
		     (uta->y0 + y) * ART_UTILE_SIZE + ART_UTA_BBOX_Y0(ub),
		     ART_UTA_BBOX_X1(ub)-ART_UTA_BBOX_X0(ub),
		     ART_UTA_BBOX_Y1(ub)-ART_UTA_BBOX_Y0(ub));
	}
      }
  }
}
